#!/bin/bash
# This is a very simple script which makes it possible to make a
# VPN connection to a Draytek router (tested on Vigor 2910)
#
# Router configuration:
# click [VPN and Remote Access]
# 
# click [IPSec General Setup]
# Enter Pre-Shared Key (use long random key).
# check at least AES (it seems to work ok if all including AH are checked).
# click [OK]
# 
# click [Remote Dial-in User]
# click index of unused entry (Status X).
# check Enable this account
# IPSec Tunnel should be checked.
# click [OK]
# 
# click [Remote Access Control]
# check Enable IPSec VPN Service
# click [OK]
# 
# Router will reboot and should be ready to accept connection.
# 
# Script changes needed:
# Change PSK to Pre Shared key as above.
# If local ethernet device isn't eth0 change line beginning with GW1
# Change NET2 to remote net inside router
# Change xxx.dnsalias.com on line beginning with GW2
# 
# Script is made to both take up and down connection.
# If racoon is running script takes down connection otherwise up.
# 
# ping on ip inside NET2 initiates key exchange between racoon and router
# and after a short delay connection should be fully up.
# 
# Check syslog for racoon status.
#
    
if [ "`pidof racoon`" ]; then
  killall racoon
  sleep 2
  setkey -F -FP
  rm /var/racoon/vpn-*
  exit
fi

PSK='secret secret secret'
GW1=`ifconfig eth0 | sed -n 's/.*addr:\([0-9\.]*\) .*/\1/p'`
NET2=192.168.88.0/24
GW2=`nslookup xxx.dnsalias.com 2>/dev/null | tail -n 1 |\
  sed -n '$s/.*: \([0-9\.]*\) .*/\1/p'` 
if ! [ "$GW2" ]; then
  echo "Couldn't resolve GW2"
  exit
fi
echo "GW1: $GW1"
echo "NET2: $NET2"
echo "GW2: $GW2"

cat >/var/racoon/vpn-racoon.conf <<...
path pre_shared_key "/var/racoon/vpn-psk.txt";
listen {
   isakmp ${GW1};
   isakmp_natt ${GW1}[4500];
}
remote ${GW2} {
   exchange_mode main;
   nat_traversal on;
   ike_frag on;
   mode_cfg on;
   proposal {
      encryption_algorithm aes;
      hash_algorithm sha1;
      authentication_method pre_shared_key;
      dh_group modp1024;
   }
}
sainfo anonymous {
   pfs_group modp768;
   encryption_algorithm aes;
   authentication_algorithm hmac_sha1;
   compression_algorithm deflate;
}
...

echo -n >/var/racoon/vpn-psk.txt
chmod 0600 /var/racoon/vpn-psk.txt 
echo "${GW2} ${PSK}" >>/var/racoon/vpn-psk.txt

cat >/var/racoon/vpn-setkey.conf <<...
flush;
spdflush;
spdadd ${GW1}/32 ${NET2} any -P out ipsec
           esp/tunnel/${GW1}-${GW2}/require;
spdadd ${NET2} ${GW1}/32 any -P in ipsec
           esp/tunnel/${GW2}-${GW1}/require;
...

setkey -f /var/racoon/vpn-setkey.conf
racoon -f /var/racoon/vpn-racoon.conf
